import "Turbine.UI";
import "Turbine.UI.Lotro";

-- A stretchable window containing a quickslot.
-- It mimics a control (providing a SetParent function) by superimposing itself in front of its parent window and moving with it.
Quickslot = class ( Turbine.UI.Window );

function Quickslot:Constructor()
    Turbine.UI.Window.Constructor(self);
    
    self.quickslot = Turbine.UI.Lotro.Quickslot();
	self.quickslot:SetBackColor( Turbine.UI.Color( 1, .25, .25, .25 ) );
    self.quickslot:SetParent(self);
    self.quickslot:SetPosition(-3, -3);
    self.quickslot.ShortcutChanged = function()
        PTweety.BuffMonitor.Utils.ExecuteCallback(self, "ShortcutChanged");
    end;
    self.quickslot.MouseDown = function()
        self.mouseDown = true;
    end
    self.quickslot.MouseMove = function()
        if (self.mouseDown) then
            self.mouseMoved = true;
        end
    end
    self.quickslot.MouseUp = function()
        self.mouseDown = false;
        if (self.mouseMoved and not self.locked) then
            self.quickslot:SetShortcut(Turbine.UI.Lotro.Shortcut(0, nil));
        end
    end

    self:SetSize(32, 32);
    self.position = { 0, 0 };
    self.visible = true;
    
    self.ParentPositionChanged = function()
        self:SetPosition(unpack(self.position));
    end
    self.ParentVisibleChanged = function()
        self:SetVisible(self.parentWindow:IsVisible());
        self:SetPosition(unpack(self.position));
    end
    self.ParentActivated = function()
        -- Need to Activate ourselves so we will remain in front of the EffectWindow,
        -- even though we're on the same ZOrder layer.
        self:Activate();
    end

    -- When the screen resolution changes, all stretched windows need to be restretched:
    self.DisplaySizeChanged = function()
        self:SetSize(self.width, self.height);
    end
    PTweety.BuffMonitor.Utils.AddCallback(Turbine.UI.Display, "SizeChanged", self.DisplaySizeChanged);
end

function Quickslot:GetShortcut()
    return self.quickslot:GetShortcut();
end

function Quickslot:SetShortcut(shortcut)
    self.quickslot:SetShortcut(shortcut);
end

function Quickslot:SetLocked(locked)
    self.locked = locked;
    self.quickslot:SetAllowDrop(not locked);
end

function Quickslot:SetSize(width, height)
    self.width, self.height = width, height;
    self:SetStretchMode(0);
    Turbine.UI.Window.SetSize(self, 32, 32);
    self:SetStretchMode(1);
    Turbine.UI.Window.SetSize(self, width, height);
end

function Quickslot:SetParent(parent)
    if (parent == self.parent) then
        return;
    end
    self.parent = parent;
    -- Wait a couple of frames to attach the event handlers to the containing window.
    -- We have to do this because the 'parent' control may not yet be in a window.
    if (parent) then
        self:SetWantsUpdates(true);
        self.skipFrames = 1;
    else
        self:SetVisible(false);
        self:SetWantsUpdates(false);
    end
end

function Quickslot:Update()
    if (self.skipFrames > 0) then
        self.skipFrames = self.skipFrames - 1;
        return;
    end
    self:SetWantsUpdates(false); -- run only once, 2 frames after SetParent() is called

    -- Find the top-level window containing the parent control, so we can attach event handlers to it
    if (self.parentWindow) then
        PTweety.BuffMonitor.Utils.RemoveCallback(self.parentWindow, "PositionChanged", self.ParentPositionChanged);
        PTweety.BuffMonitor.Utils.RemoveCallback(self.parentWindow, "VisibleChanged", self.ParentVisibleChanged);
        PTweety.BuffMonitor.Utils.RemoveCallback(self.parentWindow, "AfterActivation", self.ParentActivated);
    end
    self.parentWindow = self.parent;
    while (self.parentWindow:GetParent()) do
        self.parentWindow = self.parentWindow:GetParent();
    end
    PTweety.BuffMonitor.Utils.AddCallback(self.parentWindow, "PositionChanged", self.ParentPositionChanged);
    PTweety.BuffMonitor.Utils.AddCallback(self.parentWindow, "VisibleChanged", self.ParentVisibleChanged);
    PTweety.BuffMonitor.Utils.AddCallback(self.parentWindow, "AfterActivation", self.ParentActivated);
    
    -- Update position and visibility to match that of parent window
    self.ParentPositionChanged();
    self.ParentVisibleChanged();
    
    -- Bring this window just in front of the parent window
    self:SetVisible(self.parentWindow:IsVisible());
    self:SetZOrder(self.parentWindow:GetZOrder());
    self:Activate();
end

function Quickslot:SetPosition(left, top)
    self.position = { left, top };
    if (self.parent) then
        left, top = self.parent:PointToScreen(left, top);
    end
    Turbine.UI.Window.SetPosition(self, left, top);
end

function Quickslot:Destroy()
    if (self.parentWindow) then
        PTweety.BuffMonitor.Utils.RemoveCallback(self.parentWindow, "PositionChanged", self.ParentPositionChanged);
        PTweety.BuffMonitor.Utils.RemoveCallback(self.parentWindow, "VisibleChanged", self.ParentVisibleChanged);
        PTweety.BuffMonitor.Utils.RemoveCallback(self.parentWindow, "AfterActivation", self.ParentActivated);
    end
    PTweety.BuffMonitor.Utils.RemoveCallback(Turbine.UI.Display, "SizeChanged", self.DisplaySizeChanged);
    self:Close();
end
